# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2017 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from odoo import api, fields, models, _
from odoo.addons.purchase_requisition.models.purchase_requisition import ProcurementRule as PR
from odoo.addons.stock.models.procurement import ProcurementGroup as PG
from odoo.http import request
from odoo.exceptions import UserError, ValidationError
import logging

logger = logging.getLogger(__name__)


class PurchaseRequisition(models.Model):
    _inherit = 'purchase.requisition'

    @api.model
    def create(self, values):
        """ Look mrp/models/procurement.py +24 and you will understand universe"""
        res = super(PurchaseRequisition,
                    self.with_context(default_user_id=request.uid))\
            .create(values)
        return res


class PurchaseOrder(models.Model):
    _inherit = 'purchase.order'

    is_opened = fields.Boolean('Is opened?')
    quantity_per_container = fields.Integer('Quantity per container')
    validity_po = fields.Date('Validity of the purchase order')
    replacement_previous_po = fields.Date('Replacement previous purchase order')
    currency_type = fields.Char('Currency type')
    price_validity = fields.Date('Price validity')
    payment_condition = fields.Char('Payment condition')
    max_permissible_ppm = fields.Integer('Max permissible ppm')
    mo_uom = fields.Char('MO UOM')
    economic_level = fields.Char('EL')
    er_field = fields.Char('ER')
    effectiveness = fields.Char('Effectiviness')
    l_level_plane_change = fields.Char('L level / Plane change')

    @api.model
    def create(self, values):
        if values.get('order_line'):
            ppids = [
                i['product_id'] for i
                in self.resolve_2many_commands('order_line', values['order_line'])
            ]
            pps = self.env['product.product'].browse(ppids)
            ptids = pps.mapped('product_tmpl_id').mapped('picking_type_id')
            if len(ptids) == 1 and ptids[0]:
                picking_type_id = ptids[0].id
                values['picking_type_id'] = picking_type_id
        res = super(PurchaseOrder, self).create(values)
        return res

    @api.constrains('economic_level')
    def economic_level_validate(self):
        for rec in self:
            if rec.is_opened and rec.economic_level:
                economic_level = rec.economic_level
                if len(economic_level) == 0:
                    continue
                if len(economic_level) == 6 and economic_level.isdigit() and int(economic_level[0:2]) <= 12:
                    rec.economic_level = economic_level[0:2] + '/' + economic_level[2:]
                    continue
                if len(economic_level) == 7 and (economic_level[0:2] + economic_level[3:]).isdigit() and int(economic_level[0:2]) <= 12 and \
                        economic_level[2] == '/':
                    continue
                raise ValidationError(_('Check the format of the %s field it must be MM/YYYY') % 'economic level')

    @api.constrains('effectiveness')
    def effectiveness_validate(self):
        for rec in self:
            if rec.is_opened and rec.effectiveness:
                effectiveness = rec.effectiveness
                if len(effectiveness) == 0:
                    continue
                if len(effectiveness) == 6 and effectiveness.isdigit() and int(effectiveness[0:2]) <= 12:
                    rec.effectiveness = effectiveness[0:2] + '/' + effectiveness[2:]
                    continue
                if len(effectiveness) == 7 and (effectiveness[0:2] + effectiveness[3:]).isdigit() and \
                    int(effectiveness[0:2]) <= 12 and effectiveness[2] == '/':
                    continue
                raise ValidationError(_('Check the format of the %s field it must be MM/YYYY') % (_('effectiveness')))

    @api.model
    def create(self, vals):
        if vals.get('name', 'New') == 'New' and vals.get('is_opened', False):
            vals['name'] = self.env['ir.sequence'].next_by_code('opened.purchase.order') or '/'
        return super(PurchaseOrder, self).create(vals)

    @api.multi
    def unlink(self):
        if self.is_opened:
            self.button_cancel()
        return super(PurchaseOrder, self).unlink()

class ProcurementGroup(models.Model):
    _inherit = "procurement.group"

    def _register_hook(self):
        PG._patch_method('run', run)
        logger.info('Patched run from ProcurementGroup')
        res = super(ProcurementGroup, self)._register_hook()
        return res

    @api.model
    def _get_rule(self, product_id, location_id, values):
        res = super()._get_rule(product_id, location_id, values)
        if not res:
            if self._is_subcontracted_service(product_id):
                rule_id = product_id.product_tmpl_id.subcontracting_service_proc_rule_id
                if rule_id:
                    return rule_id
        return res


class ProcurementRule(models.Model):
    _inherit = 'procurement.rule'

    def _prepare_purchase_order(self, product_id, product_qty, product_uom, origin, values, partner):
        res = super(ProcurementRule, self)._prepare_purchase_order(product_id, product_qty, product_uom, origin, values, partner)
        pti = product_id.product_tmpl_id.picking_type_id
        if pti:
            res['picking_type_id'] = pti.id
        return res

    def _register_hook(self):
        PR._patch_method('_run_buy', _run_buy)
        logger.info('Patched _run_buy from ProcurementRule')
        res = super(ProcurementRule, self)._register_hook()
        return res


@api.multi
def _run_buy(self, product_id, product_qty, product_uom, location_id, name, origin, values):
    if product_id.purchase_requisition != 'tenders':
        return super(PR, self)._run_buy(product_id, product_qty, product_uom, location_id, name, origin, values)
    values = self.env['purchase.requisition']._prepare_tender_values(product_id, product_qty, product_uom, location_id, name, origin, values)
    values['picking_type_id'] = product_id.product_tmpl_id.picking_type_id.id or self.picking_type_id.id
    self.env['purchase.requisition'].create(values)
    return True


@api.model
def run(self, product_id, product_qty, product_uom, location_id, name, origin, values):
    values.setdefault('company_id', self.env['res.company']._company_default_get('procurement.group'))
    values.setdefault('priority', '1')
    values.setdefault('date_planned', fields.Datetime.now())
    rule = self._get_rule(product_id, location_id, values)

    product_str = '#{}({})'.format(product_id.id, product_id.name)
    location_str = '#{}({})'.format(location_id.id, location_id.name)
    if not rule:
        logger.warning('Processing rule for product %s, location %s, values %s' % (product_id, location_id, values))
        raise UserError(_('No procurement rule found. Please verify the configuration of your routes') + '\n Product: {}, Location: {}'.format(product_str, location_str))

    getattr(rule, '_run_%s' % rule.action)(product_id, product_qty, product_uom, location_id, name, origin, values)
    return True
